@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2012 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __SIEVE_COMMANDS_H
@@ -13,8 +13,14 @@
  * Argument definition
  */
 
+enum sieve_argument_flag {
+	/* More than one of this (type of) tagged argument is allowed */
+	SIEVE_ARGUMENT_FLAG_MULTIPLE = (1 << 0)
+};
+
 struct sieve_argument_def {
 	const char *identifier;
+	enum sieve_argument_flag flags;
 
 	bool (*is_instance_of)
 		(struct sieve_validator *valdtr, struct sieve_command *cmd,
@@ -63,9 +69,22 @@ struct sieve_argument {
 /* Error handling */
 
 #define sieve_argument_validate_error(validator, arg_node, ...) \
-	sieve_validator_error(validator, (arg_node)->source_line, __VA_ARGS__)
+	sieve_validator_error(validator, \
+		((arg_node) == NULL ? 0 : (arg_node)->source_line), \
+		__VA_ARGS__)
 #define sieve_argument_validate_warning(validator, arg_node, ...) \
-	sieve_validator_warning(validator, (arg_node)->source_line, __VA_ARGS__)
+	sieve_validator_warning(validator, \
+		((arg_node) == NULL ? 0 : (arg_node)->source_line), \
+		__VA_ARGS__)
+
+#define sieve_argument_generate_error(gentr, arg_node, ...) \
+	sieve_generator_error(gentr, \
+		((arg_node) == NULL ? 0 : (arg_node)->source_line), \
+		__VA_ARGS__)
+#define sieve_argument_generate_warning(gentr, arg_node, ...) \
+	sieve_generator_warning(gentr, \
+		((arg_node) == NULL ? 0 : (arg_node)->source_line), \
+		__VA_ARGS__)
 
 /* Argument API */
 
@@ -109,7 +128,7 @@ struct sieve_command_def {
 	enum sieve_command_type type;
 
 	/* High-level command syntax */
-	int positional_arguments;
+	int positional_args;
 	int subtests;
 	bool block_allowed;
 	bool block_required;
@@ -159,8 +178,6 @@ struct sieve_command {
 	( (cmd)->def == &(definition) )
 #define sieve_command_identifier(cmd) \
 	( (cmd)->def->identifier )
-#define sieve_command_type_name(cmd) \
-	( sieve_command_def_type_name((cmd)->def) )
 
 #define sieve_commands_equal(cmd1, cmd2) \
 	( (cmd1) != NULL && (cmd2) != NULL && (cmd1)->def == (cmd2)->def )
@@ -174,6 +191,8 @@ struct sieve_command *sieve_command_create
 
 const char *sieve_command_def_type_name
 	(const struct sieve_command_def *cmd_def);
+const char *sieve_command_type_name
+	(const struct sieve_command *cmd);
 
 struct sieve_command *sieve_command_prev
 	(struct sieve_command *cmd);
@@ -194,14 +213,22 @@ bool sieve_command_block_exits_unconditionally
 /* Error handling */
 
 #define sieve_command_validate_error(validator, context, ...) \
-	sieve_validator_error(validator, (context)->ast_node->source_line, __VA_ARGS__)
+	sieve_validator_error(validator, \
+		((context) == NULL ? 0 : (context)->ast_node->source_line), \
+		__VA_ARGS__)
 #define sieve_command_validate_warning(validator, context, ...) \
-	sieve_validator_warning(validator, (context)->ast_node->source_line, __VA_ARGS__)
+	sieve_validator_warning(validator, \
+		((context) == NULL ? 0 : (context)->ast_node->source_line), \
+		__VA_ARGS__)
 
 #define sieve_command_generate_error(gentr, context, ...) \
-	sieve_generator_error(gentr, (context)->ast_node->source_line, __VA_ARGS__)
+	sieve_generator_error(gentr, \
+		((context) == NULL ? 0 : (context)->ast_node->source_line), \
+		__VA_ARGS__)
 #define sieve_command_generate_warning(gentr, context, ...) \
-	sieve_generator_warning(gentr, (context)->ast_node->source_line, __VA_ARGS__)
+	sieve_generator_warning(gentr, \
+		((context) == NULL ? 0 : (context)->ast_node->source_line), \
+		__VA_ARGS__)
 
 /* Utility macros */
 
