@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2013 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __SIEVE_GENERATOR_H
@@ -75,7 +75,7 @@ const void *sieve_generator_extension_get_context
 struct sieve_jumplist {
 	pool_t pool;
 	struct sieve_binary_block *block;
-	ARRAY_DEFINE(jumps, sieve_size_t);
+	ARRAY(sieve_size_t) jumps;
 };
 
 struct sieve_jumplist *sieve_jumplist_create
