@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __SIEVE_RESULT_H
@@ -20,7 +20,8 @@ struct sieve_side_effects_list;
 struct sieve_result;
 
 struct sieve_result *sieve_result_create
-	(struct sieve_instance *svinst, const struct sieve_message_data *msgdata,
+	(struct sieve_instance *svinst,
+		const struct sieve_message_data *msgdata,
 		const struct sieve_script_env *senv);
 
 void sieve_result_ref(struct sieve_result *result);
@@ -142,17 +143,23 @@ void sieve_result_set_failure_action
  * Result execution
  */
 
-int sieve_result_implicit_keep(struct sieve_result *result,
-	struct sieve_error_handler *ehandler);
+int sieve_result_implicit_keep
+	(struct sieve_result *result,
+		struct sieve_error_handler *ehandler,
+		enum sieve_execute_flags flags,
+		bool success);
 
 void sieve_result_mark_executed(struct sieve_result *result);
 
 int sieve_result_execute
 	(struct sieve_result *result, bool *keep,
-		struct sieve_error_handler *ehandler);
+		struct sieve_error_handler *ehandler,
+		enum sieve_execute_flags flags);
 
 bool sieve_result_executed(struct sieve_result *result);
 
+bool sieve_result_executed_delivery(struct sieve_result *result);
+
 /*
  * Result evaluation
  */
