@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2013 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __SIEVE_TYPES_H
@@ -21,6 +21,7 @@ struct sieve_binary;
 struct sieve_message_data;
 struct sieve_script_env;
 struct sieve_exec_status;
+struct sieve_trace_log;
 
 /*
  * System environment
@@ -28,17 +29,43 @@ struct sieve_exec_status;
 
 enum sieve_flag {
 	/* Relative paths are resolved to HOME */
-	SIEVE_FLAG_HOME_RELATIVE = (1 << 0),
+	SIEVE_FLAG_HOME_RELATIVE = (1 << 0)
+};
+
+/* Sieve evaluation can be performed at various different points as messages
+   are processed. */
+enum sieve_env_location {
+	/* Unknown */
+	SIEVE_ENV_LOCATION_UNKNOWN = 0,
+	/* "MDA" - evaluation is being performed by a Mail Delivery Agent */
+	SIEVE_ENV_LOCATION_MDA,
+	/* "MTA" - the Sieve script is being evaluated by a Message Transfer Agent */
+	SIEVE_ENV_LOCATION_MTA,
+	/* "MS"  - evaluation is being performed by a Message Store */
+	SIEVE_ENV_LOCATION_MS
+};
+
+/* The point relative to final delivery where the Sieve script is being
+   evaluated. */
+enum sieve_delivery_phase {
+	SIEVE_DELIVERY_PHASE_UNKNOWN = 0,
+	SIEVE_DELIVERY_PHASE_PRE,
+	SIEVE_DELIVERY_PHASE_DURING,
+	SIEVE_DELIVERY_PHASE_POST,
 };
 
 struct sieve_environment {
 	const char *hostname;
-	const char *base_dir;
+	const char *domainname;
 
+	const char *base_dir;
 	const char *username;
 	const char *home_dir;
+	const char *temp_dir;
 
 	enum sieve_flag flags;
+	enum sieve_env_location location;
+	enum sieve_delivery_phase delivery_phase;
 };
 
 /*
@@ -58,15 +85,13 @@ enum sieve_error {
 	SIEVE_ERROR_NONE = 0,
 
 	/* Temporary internal error */
-	SIEVE_ERROR_TEMP_FAIL,
+	SIEVE_ERROR_TEMP_FAILURE,
 	/* It's not possible to do the wanted operation */
 	SIEVE_ERROR_NOT_POSSIBLE,
 	/* Invalid parameters (eg. script name not valid) */
 	SIEVE_ERROR_BAD_PARAMS,
 	/* No permission to do the request */
-	SIEVE_ERROR_NO_PERM,
-	/* Out of disk space */
-	SIEVE_ERROR_NO_SPACE,
+	SIEVE_ERROR_NO_PERMISSION,
 	/* Out of disk space */
 	SIEVE_ERROR_NO_QUOTA,
 	/* Item (e.g. script or binary) cannot be found */
@@ -92,6 +117,8 @@ enum sieve_compile_flags {
 	SIEVE_COMPILE_FLAG_UPLOADED = (1<<1),
 	/* Script is being activated (usually through ManageSieve) */
 	SIEVE_COMPILE_FLAG_ACTIVATED = (1<<2),
+	/* Compiled for environment with no access to envelope */
+	SIEVE_COMPILE_FLAG_NO_ENVELOPE = (1<<3)
 };
 
 /*
@@ -113,11 +140,15 @@ struct sieve_message_data {
  * Runtime flags
  */
 
-enum sieve_runtime_flags {
+enum sieve_execute_flags {
 	/* No global extensions are allowed
 	 *  (as marked by sieve_global_extensions setting)
 	 */
-	SIEVE_RUNTIME_FLAG_NOGLOBAL = (1<<0)
+	SIEVE_EXECUTE_FLAG_NOGLOBAL = (1<<0),
+	/* Do not execute (implicit keep) at the end */
+	SIEVE_EXECUTE_FLAG_DEFER_KEEP = (1<<1),
+	/* There is no envelope */
+	SIEVE_EXECUTE_FLAG_NO_ENVELOPE = (1<<2)
 };
 
 /*
@@ -125,7 +156,7 @@ enum sieve_runtime_flags {
  */
 
 typedef enum {
-	SIEVE_TRLVL_NONE,
+	SIEVE_TRLVL_NONE = 0,
 	SIEVE_TRLVL_ACTIONS,
 	SIEVE_TRLVL_COMMANDS,
 	SIEVE_TRLVL_TESTS,
@@ -149,9 +180,6 @@ struct sieve_trace_config {
  */
 
 struct sieve_script_env {
-	/* Logging related */
-	const char *action_log_format;
-
 	/* Mail-related */
 	struct mail_user *user;
 	const char *default_mailbox;
@@ -166,27 +194,42 @@ struct sieve_script_env {
 	/* Callbacks */
 
 	/* Interface for sending mail */
-	void *(*smtp_open)
-		(const struct sieve_script_env *senv, const char *destination,
-			const char *return_path, FILE **file_r);
-	bool (*smtp_close)(const struct sieve_script_env *senv, void *handle);
+	void *(*smtp_start)
+		(const struct sieve_script_env *senv, const char *return_path);
+	/* Add a new recipient */
+	void (*smtp_add_rcpt)	
+		(const struct sieve_script_env *senv, void *handle,
+			const char *address);
+	/* Get an output stream where the message can be written to. The recipients
+	   must already be added before calling this. */
+	struct ostream *(*smtp_send)
+		(const struct sieve_script_env *senv, void *handle);
+	/* Abort the SMTP transaction after smtp_send() is already issued */
+	void (*smtp_abort)
+		(const struct sieve_script_env *senv, void *handle);
+	/* Returns 1 on success, 0 on permanent failure, -1 on temporary failure. */
+	int (*smtp_finish)
+		(const struct sieve_script_env *senv, void *handle,
+			const char **error_r);
 
 	/* Interface for marking and checking duplicates */
-	int (*duplicate_check)
+	bool (*duplicate_check)
 		(const struct sieve_script_env *senv, const void *id, size_t id_size);
 	void (*duplicate_mark)
 		(const struct sieve_script_env *senv, const void *id, size_t id_size,
 			time_t time);
+	void (*duplicate_flush)
+		(const struct sieve_script_env *senv);
 
 	/* Interface for rejecting mail */
-	int (*reject_mail)(const struct sieve_script_env *senv, const char *recipient,
-			const char *reason);
+	int (*reject_mail)(const struct sieve_script_env *senv,
+		const char *recipient, const char *reason);
 
 	/* Execution status record */
 	struct sieve_exec_status *exec_status;
 
 	/* Runtime trace*/
-	struct ostream *trace_stream;
+	struct sieve_trace_log *trace_log;
 	struct sieve_trace_config trace_config;
 };
 
@@ -198,11 +241,13 @@ struct sieve_script_env {
  */
 
 struct sieve_exec_status {
-	bool message_saved;
-	bool message_forwarded;
-	bool tried_default_save;
-	bool keep_original;
 	struct mail_storage *last_storage;
+
+	unsigned int message_saved:1;
+	unsigned int message_forwarded:1;
+	unsigned int tried_default_save:1;
+	unsigned int keep_original:1;
+	unsigned int store_failed:1;
 };
 
 /*
@@ -210,10 +255,11 @@ struct sieve_exec_status {
  */
 
 enum sieve_execution_exitcode {
-	SIEVE_EXEC_OK          = 1,
-	SIEVE_EXEC_FAILURE     = 0,
-	SIEVE_EXEC_BIN_CORRUPT = -1,
-	SIEVE_EXEC_KEEP_FAILED = -2
+	SIEVE_EXEC_OK           = 1,
+	SIEVE_EXEC_FAILURE      = 0,
+	SIEVE_EXEC_TEMP_FAILURE = -1,
+	SIEVE_EXEC_BIN_CORRUPT  = -2,
+	SIEVE_EXEC_KEEP_FAILED  = -3
 };
 
 #endif /* __SIEVE_TYPES_H */
